import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import * as ef from "@typespec/emitter-framework/typescript";
export function getPageSettingsTypeRefkey(operation) {
  return ay.refkey(operation, "page-settings");
}
export function getPageSettingProperties(pagingOperation) {
  const definedSettings = pagingOperation.input;
  // Only accept these page settings
  const acceptedSettings = ["continuationToken", "offset", "pageSize", "pageIndex"];
  const settingProperties = [];
  for (const key in definedSettings) {
    const property = definedSettings[key];
    if (acceptedSettings.includes(key) && !!property) {
      settingProperties.push(property);
    }
  }
  return settingProperties;
}
export function PageSettingsDeclaration(props) {
  const namePolicy = ts.useTSNamePolicy();
  const interfaceName = namePolicy.getName(props.operation.operation.name + "PageSettings", "interface");
  const settingProperties = getPageSettingProperties(props.pagingOperation);
  return _$createComponent(ts.InterfaceDeclaration, {
    "export": true,
    name: interfaceName,
    get refkey() {
      return getPageSettingsTypeRefkey(props.operation);
    },
    get children() {
      return _$createComponent(ay.For, {
        each: settingProperties,
        line: true,
        children: parameter => _$createComponent(ts.InterfaceMember, {
          get name() {
            return parameter.property.name;
          },
          optional: true,
          get type() {
            return _$createComponent(ef.TypeExpression, {
              get type() {
                return parameter.property.type;
              }
            });
          }
        })
      });
    }
  });
}