import { createComponent as _$createComponent, memo as _$memo } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import { List, StatementList, code, refkey } from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { Reference } from "@alloy-js/typescript";
import { FunctionDeclaration } from "@typespec/emitter-framework/typescript";
import * as cl from "@typespec/http-client";
import { getClientcontextDeclarationRef } from "../../client-context/client-context-declaration.js";
import { HttpRequestOptions } from "../../http-request-options.js";
import { HttpRequest } from "../../http-request.js";
import { getOperationOptionsParameterRefkey, getOperationParameters } from "../../operation-parameters.js";
export function getHttpRequestSendRefkey(httpOperation) {
  return refkey(httpOperation, "http-request-send");
}
export function getHttpRequestSendParams(httpOperation) {
  const clientLibrary = cl.useClientLibrary();
  const client = clientLibrary.getClientForOperation(httpOperation);
  const clientContextInterfaceRef = getClientcontextDeclarationRef(client);
  const signatureParams = [{
    name: "client",
    type: clientContextInterfaceRef,
    refkey: ay.refkey(client, "client")
  }, ...getOperationParameters(httpOperation)];
  const parameters = [...signatureParams];
  // re-correct the `options` parameter to Record<string, any> to accept both paging and non-paging options
  parameters[parameters.length - 1] = {
    name: "options",
    type: "Record<string, any>",
    refkey: getOperationOptionsParameterRefkey(httpOperation),
    optional: true
  };
  return parameters;
}
export function HttpRequestSend(props) {
  const httpOperation = props.httpOperation;
  const operationUrlRefkey = refkey();
  const requestOptionsRefkey = refkey();
  const verb = props.httpOperation.verb;
  const namePolicy = ts.useTSNamePolicy();
  const functionName = namePolicy.getName(httpOperation.operation.name + "Send", "function");
  const parameters = getHttpRequestSendParams(httpOperation);
  return _$createComponent(FunctionDeclaration, {
    name: functionName,
    async: true,
    get refkey() {
      return getHttpRequestSendRefkey(httpOperation);
    },
    parametersMode: "replace",
    parameters: parameters,
    get children() {
      return _$createComponent(List, {
        get children() {
          return _$createComponent(StatementList, {
            get children() {
              return [_$createComponent(HttpRequest.Url, {
                get httpOperation() {
                  return props.httpOperation;
                },
                refkey: operationUrlRefkey
              }), _$createComponent(HttpRequestOptions, {
                get httpOperation() {
                  return props.httpOperation;
                },
                refkey: requestOptionsRefkey
              }), _$memo(() => code`return await client.pathUnchecked(${_$createComponent(Reference, {
                refkey: operationUrlRefkey
              })}).${verb}(${_$createComponent(Reference, {
                refkey: requestOptionsRefkey
              })});`)];
            }
          });
        }
      });
    }
  });
}