import { createComponent as _$createComponent, memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { code, List, refkey } from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { FunctionDeclaration } from "@typespec/emitter-framework/typescript";
import { httpRuntimeTemplateLib } from "../../external-packages/ts-http-runtime.js";
import { HttpResponses } from "../../http-response.js";
import { getOperationOptionsTypeRefkey } from "../../operation-options.js";
import { getOperationOptionsParameterRefkey } from "../../operation-parameters.js";
import { getCreateRestErrorRefkey } from "../../static-helpers/rest-error.js";
export function getHttpRequestDeserializeRefkey(httpOperation) {
  return refkey(httpOperation, "http-request-deserialize");
}
export function HttpResponseDeserialize(props) {
  const httpOperation = props.httpOperation;
  const namePolicy = ts.useTSNamePolicy();
  const functionName = namePolicy.getName(httpOperation.operation.name + "Deserialize", "function");
  const params = [{
    name: "response",
    type: httpRuntimeTemplateLib.PathUncheckedResponse
  }, {
    name: "options",
    refkey: getOperationOptionsParameterRefkey(httpOperation),
    type: getOperationOptionsTypeRefkey(httpOperation),
    optional: true
  }];
  return _$createComponent(FunctionDeclaration, {
    name: functionName,
    get refkey() {
      return getHttpRequestDeserializeRefkey(httpOperation);
    },
    parametersMode: "replace",
    parameters: params,
    get children() {
      return [code`      
      if (typeof options?.operationOptions?.onResponse === "function") {
        options?.operationOptions?.onResponse(response);
      }`, _$createComponent(List, {
        hardline: true,
        get children() {
          return [_$createComponent(HttpResponses, {
            get httpOperation() {
              return props.httpOperation;
            }
          }), _$memo(() => code`throw ${getCreateRestErrorRefkey()}(response);`)];
        }
      })];
    }
  });
}