import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { useTsp } from "@typespec/emitter-framework";
import { unpackProperty } from "../../utils/unpack-model-property.js";
import { ScalarDataTransform } from "../data-transform.js";
import { useTransformNamePolicy } from "../transform-name-policy.js";
import { JsonTransform } from "./json-transform.js";
export function JsonModelPropertyTransform(props) {
  const {
    $
  } = useTsp();
  const transformNamer = useTransformNamePolicy();
  const propertyValueType = unpackProperty(props.type);
  const transportName = transformNamer.getTransportName(props.type);
  const applicationName = transformNamer.getApplicationName(props.type);
  const targetName = props.target === "transport" ? transportName : applicationName;
  const sourceName = props.target === "transport" ? applicationName : transportName;
  const propertyValueRef = props.itemRef ? ay.code`${props.itemRef}.${sourceName}` : sourceName;
  let propertyValue;
  if ($.scalar.is(propertyValueType)) {
    propertyValue = _$createComponent(ScalarDataTransform, {
      get type() {
        return props.type;
      },
      get target() {
        return props.target;
      },
      itemRef: propertyValueRef
    });
  } else {
    propertyValue = _$createComponent(JsonTransform, {
      type: propertyValueType,
      get target() {
        return props.target;
      },
      itemRef: propertyValueRef
    });
  }
  if (props.target === "transport") {
    return _$createComponent(ay.NamePolicyContext.Provider, {
      value: {
        getName: n => n
      },
      get children() {
        return _$createComponent(ts.ObjectProperty, {
          name: targetName,
          value: propertyValue
        });
      }
    });
  }
  return [_$createComponent(ts.ObjectProperty, {
    get name() {
      return JSON.stringify(targetName);
    },
    value: propertyValue
  })];
}