import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ts from "@alloy-js/typescript";
import { TransformNamePolicyContext, useTsp } from "@typespec/emitter-framework";
import { reportDiagnostic } from "../../lib.js";
import { ContentTypeEncodingProvider } from "./content-type-encoding-provider.js";
import { JsonTransform } from "./json/json-transform.js";
import { MultipartTransform } from "./multipart/multipart-transform.js";
import { defaultTransportNameGetter } from "./transform-name-policy.js";
export function OperationTransformExpression(props) {
  const body = props.httpOperation.parameters.body;
  if (!body) {
    return;
  }
  const itemRef = body.property ? payloadApplicationNameGetter(body.property) : null;
  if (body.bodyKind === "multipart") {
    return _$createComponent(MultipartTransform, {
      body: body
    });
  }

  // TODO: Handle content types other than application/json and multipart
  // And multiple content types
  const contentType = body.contentTypes[0];
  const payloadType = body.type;
  return _$createComponent(ContentTypeEncodingProvider, {
    contentType: contentType,
    get children() {
      return _$createComponent(TransformNamePolicyContext.Provider, {
        value: {
          getTransportName: defaultTransportNameGetter,
          getApplicationName: payloadApplicationNameGetter
        },
        get children() {
          return _$createComponent(JsonTransform, {
            itemRef: itemRef,
            target: "transport",
            type: payloadType
          });
        }
      });
    }
  });
}
function payloadApplicationNameGetter(type) {
  const {
    $
  } = useTsp();
  if (typeof type.name !== "string") {
    reportDiagnostic($.program, {
      code: "symbol-name-not-supported",
      target: type
    });
    return "";
  }
  const namePolicy = ts.useTSNamePolicy();
  let name = namePolicy.getName(type.name, "object-member-data");
  if ($.modelProperty.is(type) && type.optional) {
    name = `options?.${name}`;
  }
  return name;
}